@@ -16,6 +16,12 @@ from . import InvalidStatus
 import pytz
 
 
+class ConvertionError(Exception):
+    def __init__(self, name, message):
+        self.name = name
+        self.message = message
+
+
 def datetime_rfc3339(ob):
     if ob.tzinfo:
         return ob.isoformat()
@@ -61,23 +67,48 @@ normalize_dict = {('string', None): maybe_wrap(text_),
                   ('string', 'date'): maybe_wrap(parse_date),
                   ('xxx', None): raise_error}
 
+prepare_dict = {'string': maybe_wrap(text_),
+                'number': maybe_wrap(float),
+                'integer': maybe_wrap(int),
+                'boolean': maybe_wrap(bool)}
+
 
 def jsonify_of(ob, name, type_, registry=jsonify_dict):
     try:
         convert_fn = registry[type_]
     except KeyError:
-        raise Exception("convert {} failure. unknown format {} of {}".format(name, type_, ob))
+        raise ConvertionError(name, "convert {} failure. unknown format {} of {}".format(name, type_, ob))
     return convert_fn(getattr(ob, name, None))
 
+marker = object()
+
 
 def normalize_of(ob, name, type_, registry=normalize_dict):
     try:
         convert_fn = registry[type_]
     except KeyError:
-        raise Exception("convert {} failure. unknown format {} of {}".format(name, type_, ob))
-    return convert_fn(ob.get(name))
+        raise ConvertionError(name, "convert {} failure. unknown format {} of {}".format(name, type_, ob))
+    try:
+        val = ob.get(name, marker)
+        if val is marker:
+            return val
+        return convert_fn(val)
+    except ValueError as e:
+        raise ConvertionError(name, e.args[0])
 
 
+def prepare_of(ob, name, type_, registry=prepare_dict):
+    val = ob.get(name, marker)
+    if val is marker:
+        return val
+    try:
+        convert_fn = registry[type_[0]]
+        return convert_fn(val)
+    except KeyError:
+        return val
+    except ValueError as e:
+        raise ConvertionError(name, e.args[0])
+
 def attribute_of(ob, name, type_):
     return getattr(ob, name)
 
@@ -96,12 +127,19 @@ def normalize(ob, schema, convert=normalize_of, getter=dict.get, registry=normal
     return dictify_properties(ob, schema["properties"], convert=convert, getter=getter)
 
 
-def dictify_properties(ob, properties, convert, getter):
+def prepare(ob, schema, convert=prepare_of, getter=dict.get, registry=prepare_dict):
+    convert = partial(convert, registry=registry)
+    return dictify_properties(ob, schema["properties"], convert=convert, getter=getter)
+
+
+def dictify_properties(ob, properties, convert, getter, marker=marker):
     if ob is None:
         return None
     D = {}
     for k, v in properties.items():
-        D[k] = _dictify(ob, k, v, convert, getter)
+        val = _dictify(ob, k, v, convert, getter)
+        if val is not marker:
+            D[k] = val
     return D
 
 
@@ -156,7 +194,8 @@ class ComposedModule(object):
 # objectify
 def objectify(params, schema, modellookup, strict=True):
     model_class = modellookup(schema["title"])
-    result = model_class(**objectify_propperties(params, schema["properties"], modellookup))
+    params = objectify_propperties(params, schema["properties"], modellookup)
+    result = model_class(**params)
     modellookup.pop()
     if strict:
         for k in schema["required"]:
@@ -180,6 +219,8 @@ def _objectify(params, name, schema, modellookup):
 
     if type_ == "array":
         return [_objectify_subobject(e, name, schema["items"], modellookup) for e in params.get(name, [])]
+    elif name not in params:
+        return None
     elif type_ is None:  # object
         sub_params = params.get(name)
         if sub_params is None:
@@ -200,13 +241,18 @@ def _objectify_subobject(params, name, schema, modellookup):
 
 
 class ErrorFound(Exception):  # xxx:
-    pass
+    def __init__(self, errors):
+        self.errors = errors
 
 
-def validate_all(data, validator):
+def raise_error(data, e):
+    raise e
+
+
+def validate_all(data, validator, treat_error=raise_error):
     errors = []
     for e in validator.iter_errors(data):
         errors.append(e)
     if errors:
-        raise ErrorFound(errors)
+        return treat_error(data, ErrorFound(errors))
     return data
